@@ -20,15 +20,16 @@
  *                                                                         *
  ***************************************************************************/
 
-
 #ifndef GUI_DOCKWND_REPORTVIEW_H
 #define GUI_DOCKWND_REPORTVIEW_H
 
+#include <QPointer>
 #include <QTextEdit>
 #include <QSyntaxHighlighter>
-#include <Base/Console.h>
-#include "DockWindow.h"
+
 #include "Window.h"
+#include <FCGlobal.h>
+
 
 class QTabWidget;
 
@@ -38,6 +39,7 @@ namespace DockWnd {
 
 class ReportOutput;
 class ReportHighlighter;
+class ReportOutputObserver;
 
 /** Report view containing an output window and a very simple Python console.
  * @see ReportOutput
@@ -49,15 +51,15 @@ class ReportView : public QWidget
     Q_OBJECT
 
 public:
-    ReportView( QWidget* parent = 0);
-    ~ReportView();
+    explicit ReportView( QWidget* parent = nullptr);
+    ~ReportView() override;
 
 protected:
-    void changeEvent(QEvent *e);
+    void changeEvent(QEvent *e) override;
 
 private:
     QTabWidget* tabWidget;
-    ReportOutput* tabOutput; /**< Output window */
+    ReportOutput* tabOutput; /**< Output 'Report view' window */
     PythonConsole* tabPython; /**< Python console */
 };
 
@@ -66,25 +68,27 @@ private:
  */
 class GuiExport ReportHighlighter : public QSyntaxHighlighter
 {
-public: 
-    enum Paragraph { 
-        Message  = 0, /**< normal text */
-        Warning  = 1, /**< Warning */
-        Error    = 2, /**< Error text */
-        LogText  = 3  /**< Log text */
+public:
+    enum Paragraph {
+        Message          = 0, /**< normal text */
+        Warning          = 1, /**< Warning */
+        Error            = 2, /**< Error text */
+        LogText          = 3,  /**< Log text */
+        Critical         = 4, /**< critical text */
     };
 
 public:
-    ReportHighlighter(QTextEdit* );
-    ~ReportHighlighter();
+    explicit ReportHighlighter(QTextEdit* );
+    ~ReportHighlighter() override;
 
     /** Parses the given text and highlight it in the right colors. */
-    void highlightBlock ( const QString & text );
-    /** 
+    void highlightBlock ( const QString & text ) override;
+    /**
      * Sets the current paragraph type used in ReportOutput
      * @see ReportOutput::Message
      * @see ReportOutput::Warning
      * @see ReportOutput::Error
+     * @see ReportOutput::Critical
      */
     void setParagraphType(Paragraph);
 
@@ -108,57 +112,64 @@ public:
      */
     void setErrorColor( const QColor& col );
 
+    /**
+     * Sets the text color to  \a col.
+     */
+    void setCriticalColor( const QColor& col );
+
 private:
     /** @name for internal use only */
     //@{
     Paragraph type;
-    QColor txtCol, logCol, warnCol, errCol;
+    QColor txtCol, logCol, warnCol, errCol, criticalCol;
     //@}
 };
 
-/** Output window to show messages.
- * @see Base::ConsoleObserver
+/** Output window 'Report view' to show messages.
+ * @see Base::ILogger
  * @see QTextEdit
  * \author Werner Mayer
  */
-class GuiExport ReportOutput : public QTextEdit, public WindowParameter, public Base::ConsoleObserver
+class GuiExport ReportOutput : public QTextEdit, public WindowParameter, public Base::ILogger
 {
     Q_OBJECT
 
 public:
-    ReportOutput(QWidget* parent=0);
-    virtual ~ReportOutput();
+    explicit ReportOutput(QWidget* parent=nullptr);
+    ~ReportOutput() override;
 
     /** Observes its parameter group. */
-    void OnChange(Base::Subject<const char*> &rCaller, const char * sReason);
+    void OnChange(Base::Subject<const char*> &rCaller, const char * sReason) override;
 
-    /** Writes warnings */
-    void Warning(const char * s);
-    /** Writes normal text */
-    void Message(const char * s);
-    /** Writes errors */
-    void Error  (const char * s);
-    /** Does not do anything */
-    void Log (const char * s);
+    void SendLog(const std::string& notifiername, const std::string& msg, Base::LogStyle level,
+                 Base::IntendedRecipient recipient, Base::ContentType content) override;
 
     /// returns the name for observer handling
-    const char* Name(void){return "ReportOutput";}
+    const char* Name() override {return "ReportOutput";}
 
     /** Restore the default font settings. */
     void restoreFont ();
 
-    /** Returns true whether errors are reported. */ 
+    /** Returns true whether errors are reported. */
     bool isError() const;
-    /** Returns true whether warnings are reported. */ 
+    /** Returns true whether warnings are reported. */
     bool isWarning() const;
-    /** Returns true whether log messages are reported. */ 
-    bool isLogging() const;
+    /** Returns true whether log messages are reported. */
+    bool isLogMessage() const;
+    /** Returns true whether normal messages are reported. */
+    bool isNormalMessage() const;
+    /** Returns true whether critical messages are reported. */
+    bool isCritical() const;
 
 protected:
     /** For internal use only */
-    void customEvent ( QEvent* ev );
+    void customEvent ( QEvent* ev ) override;
+    /** Handles the change of style sheets */
+    void changeEvent(QEvent *) override;
     /** Pops up the context menu with some extensions */
-    void contextMenuEvent ( QContextMenuEvent* e );
+    void contextMenuEvent ( QContextMenuEvent* e ) override;
+    /** Handle shortcut override events */
+    bool event(QEvent* event) override;
 
 public Q_SLOTS:
     /** Save the report messages into a file. */
@@ -168,14 +179,54 @@ public Q_SLOTS:
     /** Toggles the report of warnings. */
     void onToggleWarning();
     /** Toggles the report of log messages. */
-    void onToggleLogging();
+    void onToggleLogMessage();
+    /** Toggles the report of normal messages. */
+    void onToggleNormalMessage();
+    /** Toggles the report of normal messages. */
+    void onToggleCritical();
+    /** Toggles whether to show report view on warnings*/
+    void onToggleShowReportViewOnWarning();
+    /** Toggles whether to show report view on errors*/
+    void onToggleShowReportViewOnError();
+    /** Toggles whether to show report view on normal messages*/
+    void onToggleShowReportViewOnNormalMessage();
+    /** Toggles whether to show report view on normal messages*/
+    void onToggleShowReportViewOnCritical();
+    /** Toggles whether to show report view on log messages*/
+    void onToggleShowReportViewOnLogMessage();
+    /** Toggles the redirection of Python stdout. */
+    void onToggleRedirectPythonStdout();
+    /** Toggles the redirection of Python stderr. */
+    void onToggleRedirectPythonStderr();
     /** Toggles the report to go to the end if new messages appear. */
     void onToggleGoToEnd();
 
 private:
+    class Data;
+    Data* d;
     bool gotoEnd;
+    bool blockStart;
     ReportHighlighter* reportHl; /**< Syntax highlighter */
-    ParameterGrp::handle _prefs; 
+    int messageSize;
+    ParameterGrp::handle _prefs;
+};
+
+/**
+ * Observer to enable report view on warnings / errors if not already
+ * enabled.
+ */
+
+class ReportOutputObserver : public QObject
+{
+    Q_OBJECT
+
+public:
+    explicit ReportOutputObserver (ReportOutput* view);
+    bool eventFilter(QObject *obj, QEvent *event) override;
+
+protected:
+    QPointer <ReportOutput> reportView;
+    void showReportView();
 };
 
 } // namespace DockWnd
