@@ -24,24 +24,24 @@
 #ifndef GUI_VIEW_H
 #define GUI_VIEW_H
 
-#include <vector>
 #include <Base/BaseClass.h>
+#include <FCGlobal.h>
 
 namespace App
 {
 class Document;
 }
 
-namespace Gui 
+namespace Gui
 {
 class Document;
 class ViewProvider;
 
 /** Base class of all windows belonging to a document
- *  there are two ways of belonging to a document. The 
+ *  there are two ways of belonging to a document. The
  *  first way is to a fixed one. The second way is to always
  *  belonging to the Active document. that means switching every time
- *  the active document is changing. It also means that the view 
+ *  the active document is changing. It also means that the view
  *  belongs sometimes to no document at all!
  *  @see TreeView
  *  @see Gui::Document
@@ -50,7 +50,7 @@ class ViewProvider;
  */
 class GuiExport BaseView : public Base::BaseClass
 {
-    TYPESYSTEM_HEADER();
+    TYPESYSTEM_HEADER_WITH_OVERRIDE();
 
 public:
     /** View constructor
@@ -58,20 +58,20 @@ public:
      * the view will attach to the active document. Be aware! there isn't
      * always an active document!
      */
-    BaseView(Gui::Document* pcDocument=0);
+    BaseView(Gui::Document* pcDocument=nullptr);
     /** View destructor
      * Detach the view from the document, if attached!
      */
-    virtual ~BaseView();
+    ~BaseView() override;
 
 
-    /** @name methods used by the Application and the GuiDocument 
+    /** @name methods used by the Application and the GuiDocument
      */
     //@{
     /// sets the view to another document (called by Application)
     void setDocument(Gui::Document* pcDocument);
     /// is sent from the document in order to close the document
-    void onClose(void);
+    void onClose();
     //@}
 
     /// returns the document the view is attached to
@@ -79,35 +79,36 @@ public:
     /// returns the document the view is attached to
     App::Document* getAppDocument() const;
     /// indicates if the view is in passive mode
-    bool isPassive(void) const {return bIsPassive;}
+    bool isPassive() const {return bIsPassive;}
 
-    /** @name methods to overrride 
+    /** @name methods to override
      */
     //@{
     /// get called when the document is updated
-    virtual void onUpdate(void)=0;
+    virtual void onUpdate(){}
     /// get called when the document is relabeled (change of its user name)
-    virtual void onRelabel(Gui::Document *){};
+    virtual void onRelabel(Gui::Document *){}
     /// get called when the document is renamed (change of its internal name)
-    virtual void onRename(Gui::Document *){};
+    virtual void onRename(Gui::Document *){}
     /// returns the name of the view (important for messages)
-    virtual const char *getName(void) const=0;
+    virtual const char *getName() const
+    { return "Base view"; }
     /// Message handler
     virtual bool onMsg(const char* pMsg, const char** ppReturn)=0;
     /// Message handler test
     virtual bool onHasMsg(const char* pMsg) const=0;
     /// overwrite when checking on close state
-    virtual bool canClose(void){return true;}
+    virtual bool canClose(){return true;}
     /// delete itself
     virtual void deleteSelf();
     //@}
 
 protected:
     Gui::Document*  _pcDocument;
-    bool bIsDetached;
-    bool bIsPassive;
+    bool bIsDetached{false};
+    bool bIsPassive{false};
 };
 
 } // namespace Gui
 
-#endif // GUI_VIEW_H 
+#endif // GUI_VIEW_H
