@@ -1,28 +1,6 @@
 """
 Examples for a feature class and its view provider.
-
-***************************************************************************
-*   Copyright (c) 2009 Werner Mayer <wmayer[at]users.sourceforge.net>     *
-*                                                                         *
-*   This file is part of the FreeCAD CAx development system.              *
-*                                                                         *
-*   This program is free software; you can redistribute it and/or modify  *
-*   it under the terms of the GNU General Public License (GPL)            *
-*   as published by the Free Software Foundation; either version 2 of     *
-*   the License, or (at your option) any later version.                   *
-*   for detail see the LICENCE text file.                                 *
-*                                                                         *
-*   FreeCAD is distributed in the hope that it will be useful,            *
-*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
-*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
-*   GNU Library General Public License for more details.                  *
-*                                                                         *
-*   You should have received a copy of the GNU Library General Public     *
-*   License along with FreeCAD; if not, write to the Free Software        *
-*   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  *
-*   USA                                                                   *
-*                                                                         *
-***************************************************************************
+(c) 2009 Werner Mayer LGPL
 """
 
 __author__ = "Werner Mayer <wmayer@users.sourceforge.net>"
@@ -46,8 +24,6 @@ class Box(PartFeature):
 	def onChanged(self, fp, prop):
 		''' Print the name of the property that has changed '''
 		FreeCAD.Console.PrintMessage("Change property: " + str(prop) + "\n")
-		if prop == "Length" or prop == "Width" or prop == "Height":
-			fp.Shape = Part.makeBox(fp.Length,fp.Width,fp.Height)
 
 	def execute(self, fp):
 		''' Print a short message when doing a recomputation, this method is mandatory '''
@@ -78,7 +54,7 @@ class ViewProviderBox:
 
 	def setDisplayMode(self,mode):
 		''' Map the display mode defined in attach with those defined in getDisplayModes.
-		Since they have the same names nothing needs to be done. This method is optinal.
+		Since they have the same names nothing needs to be done. This method is optional.
 		'''
 		return mode
 
@@ -133,10 +109,11 @@ class ViewProviderBox:
 
 
 def makeBox():
-	FreeCAD.newDocument()
+	doc=FreeCAD.newDocument()
 	a=FreeCAD.ActiveDocument.addObject("Part::FeaturePython","Box")
 	Box(a)
 	ViewProviderBox(a.ViewObject)
+	doc.recompute()
 
 # -----------------------------------------------------------------------------
 
@@ -161,11 +138,12 @@ class ViewProviderLine:
 		return "Flat Lines"
 
 def makeLine():
-	FreeCAD.newDocument()
+	doc=FreeCAD.newDocument()
 	a=FreeCAD.ActiveDocument.addObject("Part::FeaturePython","Line")
 	Line(a)
 	#ViewProviderLine(a.ViewObject)
 	a.ViewObject.Proxy=0 # just set it to something different from None
+	doc.recompute()
 
 # -----------------------------------------------------------------------------
 
@@ -199,7 +177,7 @@ class Octahedron:
 		shell=Part.makeShell([f1,f2,f3,f4,f5,f6,f7,f8])
 		solid=Part.makeSolid(shell)
 		fp.Shape = solid
-	# helper mehod to create the faces
+	# helper method to create the faces
 	def make_face(self,v1,v2,v3):
 		wire = Part.makePolygon([v1,v2,v3,v1])
 		face = Part.Face(wire)
@@ -341,10 +319,11 @@ class ViewProviderOctahedron:
 		return None
 
 def makeOctahedron():
-	FreeCAD.newDocument()
+	doc=FreeCAD.newDocument()
 	a=FreeCAD.ActiveDocument.addObject("App::FeaturePython","Octahedron")
 	Octahedron(a)
 	ViewProviderOctahedron(a.ViewObject)
+	doc.recompute()
 
 # -----------------------------------------------------------------------------
 
@@ -384,7 +363,7 @@ class ViewProviderPoints:
 
 	def setDisplayMode(self,mode):
 		''' Map the display mode defined in attach with those defined in getDisplayModes.
-		Since they have the same names nothing needs to be done. This method is optinal.
+		Since they have the same names nothing needs to be done. This method is optional.
 		'''
 		return mode
 
@@ -439,7 +418,7 @@ class ViewProviderPoints:
 
 
 def makePoints():
-	FreeCAD.newDocument()
+	doc=FreeCAD.newDocument()
 	import Mesh
 	m=Mesh.createSphere(5.0).Points
 	import Points
@@ -456,6 +435,7 @@ def makePoints():
 	a.Points=p
 	PointFeature(a)
 	ViewProviderPoints(a.ViewObject)
+	doc.recompute()
 
 # -----------------------------------------------------------------------------
 
@@ -531,29 +511,25 @@ class ViewProviderMesh:
 
 
 def makeMesh():
-	FreeCAD.newDocument()
+	doc=FreeCAD.newDocument()
 	import Mesh
 
 	a=FreeCAD.ActiveDocument.addObject("Mesh::FeaturePython","Mesh")
 	a.Mesh=Mesh.createSphere(5.0)
 	MeshFeature(a)
 	ViewProviderMesh(a.ViewObject)
-	
+	doc.recompute()
+
 # -----------------------------------------------------------------------------
 
 class Molecule:
 	def __init__(self, obj):
 		''' Add two point properties '''
 		obj.addProperty("App::PropertyVector","p1","Line","Start point")
-		obj.addProperty("App::PropertyVector","p2","Line","End point").p2=FreeCAD.Vector(1,0,0)
+		obj.addProperty("App::PropertyVector","p2","Line","End point").p2=FreeCAD.Vector(5,0,0)
 
 		obj.Proxy = self
 
-	def onChanged(self, fp, prop):
-		if prop == "p1" or prop == "p2":
-			''' Print the name of the property that has changed '''
-			fp.Shape = Part.makeLine(fp.p1,fp.p2)
-
 	def execute(self, fp):
 		''' Print a short message when doing a recomputation, this method is mandatory '''
 		fp.Shape = Part.makeLine(fp.p1,fp.p2)
@@ -561,7 +537,6 @@ class Molecule:
 class ViewProviderMolecule:
 	def __init__(self, obj):
 		''' Set this object to the proxy object of the actual view provider '''
-		obj.Proxy = self
 		sep1=coin.SoSeparator()
 		self.trl1=coin.SoTranslation()
 		sep1.addChild(self.trl1)
@@ -572,6 +547,8 @@ class ViewProviderMolecule:
 		sep2.addChild(coin.SoSphere())
 		obj.RootNode.addChild(sep1)
 		obj.RootNode.addChild(sep2)
+		# triggers an updateData call so the assignment at the end
+		obj.Proxy = self
 
 	def updateData(self, fp, prop):
 		"If a property of the handled feature has changed we have the chance to handle this here"
@@ -590,10 +567,11 @@ class ViewProviderMolecule:
 		return None
 
 def makeMolecule():
-	FreeCAD.newDocument()
+	doc=FreeCAD.newDocument()
 	a=FreeCAD.ActiveDocument.addObject("Part::FeaturePython","Molecule")
 	Molecule(a)
 	ViewProviderMolecule(a.ViewObject)
+	doc.recompute()
 
 # -----------------------------------------------------------------------------
 
@@ -619,7 +597,7 @@ class ViewProviderCircleSet:
 
 	def updateData(self, fp, prop):
 			if prop == "Shape":
-				edges=p = fp.getPropertyByName("Shape").Edges
+				edges = fp.getPropertyByName("Shape").Edges
 				pts=[]
 				ver=[]
 				for i in edges:
@@ -650,11 +628,12 @@ def makeCircleSet():
 			y=y+0.5
 		x=x+0.5
 
-	FreeCAD.newDocument()
+	doc=FreeCAD.newDocument()
 	a=FreeCAD.ActiveDocument.addObject("App::FeaturePython","Circles")
-	c=CircleSet(a)
-	v=ViewProviderCircleSet(a.ViewObject)
+	CircleSet(a)
+	ViewProviderCircleSet(a.ViewObject)
 	a.Shape=comp
+	doc.recompute()
 
 # -----------------------------------------------------------------------------
 
@@ -666,17 +645,17 @@ class EnumTest:
 		obj.Proxy = self
 
 	def execute(self, fp):
-		return                                                                            
-                                                                               
+		return
+
 class ViewProviderEnumTest:
 	def __init__(self, obj):
-		''' Set this object to the proxy object of the actual view provider '''           
+		''' Set this object to the proxy object of the actual view provider '''
 		obj.addProperty("App::PropertyEnumeration","Enum3","","Enumeration3").Enum3=["One","Two","Three"]
 		obj.addProperty("App::PropertyEnumeration","Enum4","","Enumeration4").Enum4=["One","Two","Three"]
 		obj.Proxy = self
 
 	def updateData(self, fp, prop):
-		print "prop updated:",prop
+		print("prop updated:",prop)
 
 	def __getstate__(self):
 		return None
@@ -736,9 +715,9 @@ class DistanceBolt:
 		fp.Shape = extrude
 
 def makeDistanceBolt():
-	FreeCAD.newDocument()
+	doc=FreeCAD.newDocument()
 	bolt=FreeCAD.ActiveDocument.addObject("Part::FeaturePython","Distance_Bolt")
 	bolt.Label = "Distance bolt"
 	DistanceBolt(bolt)
 	bolt.ViewObject.Proxy=0
-
+	doc.recompute()
